/*
 * (C) COPYRIGHT International Business Machines Corp. 2004,2005
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * This sample program is provided AS IS and may be used, executed, copied
 * and modified without royalty payment by customer (a) for its own instruction and
 * study, (b) in order to develop applications designed to run with an IBM
 * WebSphere product, either for customer's own internal use or for
 * redistribution by customer, as part of such an application, in customer's
 * own products.
 *
 *  CEI/ws/code/cei.samples/src/EventService/com/ibm/events/samples/purge/SampleEventPurge.java, WAS.cei, WAS70.CEI, cf351435.01
 *  @Version 1.1 9/9/14
 *
 */
package com.ibm.events.samples.purge;

import java.rmi.RemoteException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import com.ibm.events.EventsException;
import com.ibm.events.access.EventAccess;
import com.ibm.events.access.EventAccessHome;


/**
 * This class provides sample code for an event purge.
 */
public class SampleEventPurge
{
    //~ Instance variables ...........................................

    /** The event group to use for purging events */
    private String eventGroup = null;

    /** An additional event selector for purging events */
    private String eventSelector = null;

    /**
     * The URL of the JNDI provider used to connect to the event
     * infrastructure
     */
    private String providerUrl = "corbaloc:iiop:localhost:2809";

    //~ Constructors .................................................

    /**
     * Construct the sample event purge
     */
    public SampleEventPurge()
    {
    }

    //~ Methods ......................................................

    /**
     * Main method that will take command line arguments and run the
     * event purge
     *
     * @param args Array arguments passed from the command line
     */
    public static void main(String args[])
    {
        // create an instance of the sample event purge
        SampleEventPurge eventPurge = new SampleEventPurge();

        // process the command line args
        for(int i = 0; i < args.length; i++)
        {
            if(args[i].equals("-j"))
            {
                i++;

                if(i < args.length)
                {
                    eventPurge.setProviderUrl(args[i]);
                }
            }
            else if(args[i].equals("-g"))
            {
                i++;

                if(i < args.length)
                {
                    eventPurge.setEventGroup(args[i]);
                }
            }
            else if(args[i].equals("-s"))
            {
                i++;

                if(i < args.length)
                {
                    eventPurge.setEventSelector(args[i]);
                }
            }
            else
            {
                eventPurge.displayUsage();
                System.exit(1);
            }
        }

        // run the event purge
        eventPurge.run();

        System.exit(0);
    }

    /**
     * Set the JNDI URL provider.
     *
     * @param providerUrl The JNDI URL provider
     */
    public void setProviderUrl(String providerUrl)
    {
        this.providerUrl = providerUrl;
    }

    /**
     * Display the command line usage for this sample.
     */
    public void displayUsage()
    {
        System.out.println("SampleEventPurge [-j <providerUrl>] [-g <eventGroup>] [-s <eventSelector>]");
        System.out.println("\t-j <providerUrl> : The JNDI provider URL. Such as corbaloc:iiop:localhost:2809");
        System.out.println("\t-g <eventGroup> : The event group used for purges.");
        System.out.println("\t-s <eventSelector> : The event selector used for purges.");
    }

    /**
     * Run the sample event purge
     */
    public void run()
    {
        try
        {
            // get the time that the purge started
            long startTime = System.currentTimeMillis();

            // look up the event access EJB.
            Hashtable jndiConfig = new Hashtable();
            jndiConfig.put(Context.INITIAL_CONTEXT_FACTORY,
                           "com.ibm.websphere.naming.WsnInitialContextFactory");
            jndiConfig.put(Context.PROVIDER_URL,
                           providerUrl);

            Context         initial = new InitialContext(jndiConfig);

            Object          objref = initial.lookup("ejb/com/ibm/events/access/EventAccess");

            EventAccessHome eventAccessHome = (EventAccessHome) PortableRemoteObject.narrow(objref,
                                                                                            EventAccessHome.class);

            EventAccess eventAccess = eventAccessHome.create();

            // execute the purge
            int number = eventAccess.purgeEvents(eventGroup,
                                                 eventSelector,
                                                 EventAccess.DEFAULT_PURGE_TRANSACTION_SIZE);

            // set the time that the purge finshed
            long endTime = System.currentTimeMillis();

            // display the number of events purged
            // and the time that it took to purge the events
            System.out.println(number + " Events purged in " +
                               (endTime - startTime) +
                               " milliseconds");
        }
        catch(RemoteException e)
        {
            e.printStackTrace();
        }
        catch(EventsException e)
        {
            e.printStackTrace();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Sets the event group used for event purge
     *
     * @param eventGroup The event group to use for event purge
     */
    void setEventGroup(String eventGroup)
    {
        this.eventGroup = eventGroup;
    }

    /**
     * Sets the event selector used for event purge
     *
     * @param eventSelector The event selector to use for event purge
     */
    void setEventSelector(String eventSelector)
    {
        this.eventSelector = eventSelector;
    }
}