/*
 * (C) COPYRIGHT International Business Machines Corp. 2004,2005
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * This sample program is provided AS IS and may be used, executed, copied
 * and modified without royalty payment by customer (a) for its own instruction and
 * study, (b) in order to develop applications designed to run with an IBM
 * WebSphere product, either for customer's own internal use or for
 * redistribution by customer, as part of such an application, in customer's
 * own products.
 *
 *  CEI/ws/code/cei.samples/src/EventService/com/ibm/events/samples/source/SampleEventSource.java, WAS.cei, WAS70.CEI, cf351435.01
 *  @Version 1.1 9/9/14
 *
 */
package com.ibm.events.samples.source;

import com.ibm.events.EventsException;
import com.ibm.events.emitter.Emitter;
import com.ibm.events.emitter.EmitterFactory;

import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.ComponentIdentification;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.events.cbe.ValidationException;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;

import javax.naming.Context;
import javax.naming.InitialContext;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.util.Hashtable;


/**
 * This class provides sample code for an event source.
 */
public class SampleEventSource implements Runnable
{
    //~ Instance variables .........................................................................

    /** THe emitter instance to send events through */
    private Emitter emitter = null;

    /** The event factory to create all events. */
    private EventFactory eventFactory = null;

    /** The name of the emiyter profile */
    private String emitterProfile = "com/ibm/events/configuration/emitter/Default";

    /** The name of the emiyter profile */
    private String eventFactoryJndiName = "com/ibm/events/EventFactory";

    /** The host name of the machine sending the event */
    private String hostname = "unknown";

    /** The JMS password, required if security is on */
    private String jmsPassword = null;

    /** The JMS username, required if security is on */
    private String jmsUsername = null;

    /** The message of the event */
    private String message = null;

    /** The URL of the JNDI provider used to connect to the event infrastructure */
    private String providerUrl = "corbaloc:iiop:localhost:2809";

    /** The XML file containing the event */
    private String xmlEvent = null;

    /** The number of events to send in a batch */
    private int batchSize = 1;

    /** The number of events to submit */
    private int number = 1;

    /** The number of threads to create and run emitter from */
    private int numberThreads = 1;

    /** The delay in milliseconds */
    private long delay = 0;

    /** The severity of the event */
    private short severity = -1;

    //~ Constructors ...............................................................................

    /**
     * Constructs the sample event source
     */
    public SampleEventSource()
    {
        try
        {
            hostname = InetAddress.getLocalHost().getHostName();
        }
        catch(UnknownHostException e)
        {
            e.printStackTrace();
        }
    }

    //~ Methods ....................................................................................

    /**
     * Sets the number of events to send in a batch
     *
     * @param batchSize The size of the batch
     */
    public void setBatchSize(int batchSize)
    {
        this.batchSize = batchSize;
    }

    /**
     * Sets the delay between events being submitted
     *
     * @param delay The delay in milliseconds
     */
    public void setDelay(long delay)
    {
        this.delay = delay;
    }

    /**
     * Set the emitter profile.
     *
     * @param emitterProfile The name of the emitter profile
     */
    public void setEmitterProfile(String emitterProfile)
    {
        this.emitterProfile = emitterProfile;
    }

    /**
     * Set the event factory JNDI name.
     *
     * @param eventFactoryJndiName The JNDI name for the event factory
     */
    public void setEventFactoryJndiName(String eventFactoryJndiName)
    {
        this.eventFactoryJndiName = eventFactoryJndiName;
    }

    /**
     * Set the JNDI URL provider.
     *
     * @param password The JNDI URL provider
     */
    public void setJMSPassword(String password)
    {
        this.jmsPassword = password;
    }

    /* Set the JNDI URL provider.
     *
     * @param providerUrl The JNDI URL provider
     */
    public void setJMSUsername(String username)
    {
        this.jmsUsername = username;
    }

    /**
     * Sets the message of the event
     *
     * @param message The message to use in the event
     */
    public void setMessage(String message)
    {
        this.message = message;
    }

    /**
     * Sets the number of events to submit
     *
     * @param number The number of events to submit
     */
    public void setNumber(int number)
    {
        this.number = number;
    }

    /**
     * DOCUMENT ME!
     *
     * @param numberThreads DOCUMENT ME!
     */
    public void setNumberThreads(int numberThreads)
    {
        this.numberThreads = numberThreads;
    }

    /**
     * Sets the JNDI URL provider.
     *
     * @param providerUrl The JNDI URL provider
     */
    public void setProviderUrl(String providerUrl)
    {
        this.providerUrl = providerUrl;
    }

    /**
     * Sets the severity of the event
     *
     * @param severity The severity of the event
     */
    public void setSeverity(short severity)
    {
        this.severity = severity;
    }

    /**
     * Displays the command-line usage for this sample.
     */
    public void displayUsage()
    {
        System.out.println("SampleEventSource [-j <providerUrl>] [-s <severity>] [-m <message>] [-n <number>] [-d <delay>] [-b <size>] [-u <jmsUsername> -w <jmsPassword>] [-t <numberThreads>] [-e <eventFactoryJndiName>]");
        System.out.println("\t-j <providerUrl> : The JNDI provider URL. Such as corbaloc:iiop:localhost:2809");
        System.out.println("\t-s <severity> : The severity of the event. This is a number between 0 and 70 inclusive.");
        System.out.println("\t-m <message> : The message that you want each event to contain");
        System.out.println("\t-n <number> : The number of events to send");
        System.out.println("\t-d <delay> : The delay time between sending events in milliseconds");
        System.out.println("\t-p <emitterProfile> : The name of the emitter profile used to send events");
        System.out.println("\t-x <xmlfile> : XML file containing a CommonBaseEvent");
        System.out.println("\t-u <jmsUsername> : The JMS user name");
        System.out.println("\t-w <jmsPassword> : The JMS password");
        System.out.println("\t-t <numberThreads> : The number of threads to run");
        System.out.println("\t-e <eventFactoryJndiName> : The JNDI name of the event factory to build events");
        System.out.println("\t-b <size> : The size of the event batch that you wish to send. This option is not used when specifying -x option");
    }

    /**
     * Main method that will take command line arguments and run the event source
     *
     * @param args Array arguements passed form the command line
     *
     * @throws Exception DOCUMENT ME!
     */
    public static void main(String args[]) throws Exception
    {
        // create an instance of the sample event source
        SampleEventSource eventSource = new SampleEventSource();

        // process the command line args
        for(int i = 0; i < args.length; i++)
        {
            if(args[i].equals("-j"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setProviderUrl(args[i]);
                }
            }
            else if(args[i].equals("-m"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setMessage(args[i]);
                }
            }
            else if(args[i].equals("-n"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setNumber(Integer.parseInt(args[i]));
                }
            }
            else if(args[i].equals("-s"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setSeverity(Short.parseShort(args[i]));
                }
            }
            else if(args[i].equals("-b"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setBatchSize(Integer.parseInt(args[i]));
                }
            }
            else if(args[i].equals("-d"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setDelay(Long.parseLong(args[i]));
                }
            }
            else if(args[i].equals("-p"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setEmitterProfile(args[i]);
                }
            }
            else if(args[i].equals("-x"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setXmlEvent(args[i]);
                }
            }
            else if(args[i].equals("-u"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setJMSUsername(args[i]);
                }
            }
            else if(args[i].equals("-w"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setJMSPassword(args[i]);
                }
            }
            else if(args[i].equals("-t"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setNumberThreads(Integer.parseInt(args[i]));
                }
            }
            else if(args[i].equals("-e"))
            {
                i++;

                if(i < args.length)
                {
                    eventSource.setEventFactoryJndiName(args[i]);
                }
            }
            else
            {
                eventSource.displayUsage();
                System.exit(1);
            }
        }

        eventSource.run();

        System.exit(0);
    }

    /**
     * Sets the event that was read from the XML file.
     *
     * @param xmlEvent The xml file where the event exisits.
     */
    public void setXmlEvent(String xmlEvent)
    {
        this.xmlEvent = xmlEvent;
    }

    /**
     * Runs the sample event source.
     */
    public void run()
    {
        try
        {
            Hashtable jndiConfig = new Hashtable();
            jndiConfig.put(Context.INITIAL_CONTEXT_FACTORY,
                           "com.ibm.websphere.naming.WsnInitialContextFactory");

            jndiConfig.put(Context.PROVIDER_URL,
                           providerUrl);

            Context        initial = new InitialContext(jndiConfig);

            EmitterFactory emitterFactory = (EmitterFactory) initial.lookup(emitterProfile);
            eventFactory = (EventFactory) initial.lookup(eventFactoryJndiName);

            if(jmsUsername != null)
            {
                emitter = emitterFactory.getEmitter(jmsUsername,
                                                    jmsPassword);
            }
            else
            {
                emitter = emitterFactory.getEmitter();
            }

            Thread threads[] = new Thread[numberThreads];

            for(int i = 0; i < numberThreads; i++)
            {
                threads[i] = new Thread(new EventSender());
            }

            long startTime = System.currentTimeMillis();

            for(int i = 0; i < numberThreads; i++)
            {
                threads[i].start();
            }

            for(int i = 0; i < numberThreads; i++)
            {
                threads[i].join();
            }

            long endTime      = System.currentTimeMillis();
            long duration     = endTime - startTime;
            long numberEvents = (number * batchSize) * numberThreads;
            long rate         = 0;

            if((duration / 1000) > 0)
            {
                rate = numberEvents / (duration / 1000);
            }

            System.out.println(numberEvents + " Events sent in " + duration +
                               " milliseconds at a rate of " + rate + " events/sec.");
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Reads all characters from the input stream and converts them to a String using the default
     * platform encoding.
     *
     * @param stream the input stream that contains a CommonBaseEvent XML
     *
     * @return XML string containing a CommonBaseEvent
     *
     * @throws Exception DOCUMENT ME!
     */
    private String getStringFromInputStream(InputStream stream)
                                     throws Exception
    {
        InputStreamReader isr    = new InputStreamReader(stream);
        BufferedReader    br     = new BufferedReader(isr);
        StringBuffer      cbeXml = new StringBuffer();

        long              linesRead = 0;
        long              charsRead = 0;

        String            xmlString = null;

        try
        {
            String line = br.readLine();

            while(line != null)
            {
                linesRead++;
                charsRead += line.length();
                cbeXml.append(line);
                line = br.readLine();
            }

            xmlString = cbeXml.toString();
        }
        finally
        {
            if(isr != null)
            {
                isr.close();
            }

            if(br != null)
            {
                br.close();
            }

            if(stream != null)
            {
                stream.close();
            }
        }

        return xmlString;
    }

    /**
     * Helper method to create and populate an event
     *
     * @param eventFactory The event factory to use to create the event
     *
     * @return The CommonBaseEvent populated with data
     *
     * @throws Exception If an error occurs building an event.
     */
    private CommonBaseEvent createEvent(EventFactory eventFactory)
                                 throws Exception
    {
        final CommonBaseEvent event = eventFactory.createCommonBaseEvent();

        event.setExtensionName("TestEvent");

        event.setCreationTimeAsLong(System.currentTimeMillis());

        if(severity != -1)
        {
            event.setSeverity(severity);
        }

        if(message != null)
        {
            event.setMsg(message);
        }

        Situation situation = eventFactory.createSituation();
        situation.setCategoryName(Situation.REPORT_SITUATION_CATEGORY);
        situation.setReportSituation("INTERNAL",
                                     "Suceeded");
        event.setSituation(situation);

        ComponentIdentification sourceComponent = eventFactory.createComponentIdentification();
        sourceComponent.setApplication("Sample Event Source");
        sourceComponent.setSubComponent("createEvent()");
        sourceComponent.setComponent("source.SampleEventSource");
        sourceComponent.setComponentIdType("Sample");
        sourceComponent.setLocation(hostname);
        sourceComponent.setLocationType("Hostname");
        sourceComponent.setComponentType("http://www.ibm.com/namespaces/autonomic/Tivoli/Samples");
        event.setSourceComponentId(sourceComponent);

        event.setVersion("1.0.1");

        return event;
    }

    /**
     * Reads and parse a XML file that contains a CommonBaseEvent.
     *
     * @param fileName The name of the file that contains the event.
     *
     * @return An instance of the event.
     *
     * @throws Exception If there is an error processing the XML.
     */
    private CommonBaseEvent[] loadEventFromFile(String fileName)
                                         throws Exception
    {
        CommonBaseEvent   events[]        = null;
        FileInputStream   fileInputStream = null;

        try
        {
            //Create an input stream that will read from the file
            fileInputStream = new FileInputStream(fileName);

            // pull the XML string out of the file
            String xmlDoc = getStringFromInputStream(fileInputStream);

            // Convert the XML string to an event.
            events = EventFormatter.eventsFromCanonicalXMLDoc(xmlDoc);
        }
        finally
        {
            if(fileInputStream != null)
            {
                fileInputStream.close();
            }
        }

        return events;
    }

    //~ Classes ....................................................................................

    class EventSender implements Runnable
    {
        //~ Instance variables .....................................................................

        private CommonBaseEvent events[] = null;

        //~ Constructors ...........................................................................

        EventSender() throws Exception
        {
            if(xmlEvent != null)
            {
                events = loadEventFromFile(xmlEvent);
                batchSize = events.length;
            }
            else
            {
                events = new CommonBaseEvent[batchSize];

                for(int i = 0; i < batchSize; i++)
                {
                    events[i] = createEvent(eventFactory);
                }
            }
        }

        //~ Methods ................................................................................

        public void run()
        {
            for(int i = 0; i < number; i++)
            {
                try
                {
                    for(int j = 0; j < events.length; j++)
                    {
                        //reset globalInstanceId
                        events[j].setGlobalInstanceId(null);

                        // unset sequence number
                        events[j].unsetSequenceNumber();

                        // set a new creationTime
                        events[j].setCreationTimeAsLong(System.currentTimeMillis());
                    }

                    // now send the event using the emiiter
                    emitter.sendEvents(events);

                    if(delay > 0)
                    {
                        try
                        {
                            // now sleep the specified period of time
                            Thread.sleep(delay);
                        }
                        catch(InterruptedException e)
                        {
                        }
                    }
                }
                catch(EventsException e)
                {
                    e.printStackTrace();
                }
                catch(Exception e)
                {
                    e.printStackTrace();
                }
            }
        }
    }
}