//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.appprofile.accountmanagementwar;

// General enterprise bean code.
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.FinderException;
import javax.ejb.DuplicateKeyException;

// Servlet related.
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming) for Context, InitialContext, NamingException
import javax.naming.*;  

// Miscellaneous:
import java.util.*;
import java.io.*;    
import java.text.*;

import javax.sql.*;
import java.rmi.*;
import java.sql.*;

import com.ibm.websphere.samples.appprofile.accountmanagementejb.*;

/**
 * The AccountManagementServlet is the client progeam of 
 * Application Profile Account Management sample. 
 *
 * @author Cheng-chieh (Jerry) Cheng
 * @version 1.0
 * @since 04/15/2002
 */
public class AccountManagementServlet extends HttpServlet 
{   
   private String title = "Application Profile - Account Management Sample";
   private String heading = "Application Profile - Account Management";

   private String actionPrefix = "Action";
   private String menuPrompt = "Specify a value then click an action:";

   private String action1Text = "Create multiple accounts";
   private String action2Text = "Display accounts with large balances";
   private String action3Text = "Increase APR of large accounts by 0.25";

   private String action1Description = "Number of accounts: ";
   private String action2Description = "Minimum account balance: ";
   private String action3Description = "Minimum account balance: ";

   private String action1ButtonValue = "Create";
   private String action2ButtonValue = "Show";
   private String action3ButtonValue = "Update";
   private String action4ButtonValue = "clean up database";

   private String miniBalanceText = "minimum balance";
   private String invalidBalanceMessage = "Invalid balance information";
   private String invalidAccountNumberToBeCreatedMessage = "Invalid account number to be created information";

   private String action1ResultMessage1 = "accounts have been created";
   private String action1ResultMessage2 = "Create accounts action failed";
   private String action2ResultMessage1 = "accounts meet the minimum balance requirement";
   private String action2ResultMessage2 = "No account meets the minimum balance requirement";
   private String action3ResultMessage1 = "accounts' APR have been increased";
   private String action3ResultMessage2 = "Action 3 failed";
   
   private String action1Response = "Response to " + action1ButtonValue + " action: ";
   private String action2Response = "Response to " + action2ButtonValue + " action: ";
   private String action3Response = "Response to " + action3ButtonValue + " action: ";
   private String action4Response = "Response to " + action4ButtonValue + " action: ";

   private String servletURL = "./AccountManagementServlet";
   private String imageURL = "<img src=\"/WSsamples/images/success_banner.jpg\" valign=top ALT=\"Samples Gallery\">";

   private String accountManagerName = "java:comp/env/ejb/AppProfileSample-AccountManagerHome";
   private AccountManager accountManager = null;

   private InitialContext initCtx = null;        
   private NumberFormat read = null;

   private int sampleAccountsNumber = 400;
   private double aprIncreaseRate = 0.25;
   
   private String defaultBalance = "10000";
   private String defaultMiniBalance = "10000";

   private DataSource ds = null;
   private String datasourceBinding = "jdbc/AppProfileSampleDB";
   private String tableName = "ACCOUNT";

   public void init() throws ServletException
   { 
       read = NumberFormat.getInstance();
       
       try
       {
           initCtx = new InitialContext();
           this.getAccountManager();
       }
       catch(Exception e)
       {
           e.printStackTrace();
       }
   }

   /**
    * Handle post request
    * 
   * @param req inbound HttpServletRequest object
   * @param res outbound HttpServletResponse object
   * @exception ServletException
    */
   public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
       processRequest(req,res);
   }

   /**
    * Handle get request
    * 
    * @param req inbound HttpServletRequest object
    * @param res outbound HttpServletResponse object
    * @exception ServletException
    */
   public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
       processRequest(req,res);
   }

   /**
    * The logic to handle client request
    * 
    * @param req inbound HttpServletRequest object
    * @param res outbound HttpServletResponse object
    * @exception ServletException
    */
   private void processRequest(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
       String action = "0";
       String actionSubmit = req.getParameter("action1");
       if(actionSubmit!=null) action="1";
       else
       {
           actionSubmit = req.getParameter("action2");
           if(actionSubmit!=null) action="2";
           else
           {
               actionSubmit = req.getParameter("action3");
               if(actionSubmit!=null) action="3";
           }
       }

       String balanceString = replaceSpecialCharacters(req.getParameter("balance"));
       String minibalanceString = replaceSpecialCharacters(req.getParameter("minibalance"));
       String accountNumberToBeCreatedString = replaceSpecialCharacters(req.getParameter("accountNumberToBeCreated"));
       
      // --- Prepare and send HTML response. ---
      res.setContentType("text/html");
      res.setHeader("Pragma", "no-cache");
      res.setHeader("Cache-control", "no-cache");

      PrintWriter out = res.getWriter();

      out.println("<html>");
      printPageTemplate(out);
      out.println("<head>");
      out.println("<title>" + title + "</title>");
      out.println("</head>");
      out.println("<body>");

      out.println(imageURL);
      printSampleTitle(out);
      printMenu(out,accountNumberToBeCreatedString,balanceString,minibalanceString);    
          
      if(action.equals("1"))
      {
          this.doAction1(out,accountNumberToBeCreatedString);
      }
      else if(action.equals("2"))
      {
          this.doAction2(out,balanceString);
      }
      else if(action.equals("3"))
      {
          this.doAction3(out,minibalanceString);
      }

      out.println("<HR width=100%>");
      Collection accountIdCollection = null;
      try
      {
          int size = 0;
          if(accountManager==null) this.getAccountManager();
          if(accountManager != null)
          {
              try
              {
                  size = accountManager.getAccountsNumber();
                  if(size < 0) printStatusMsg(out,"CAN NOT get total accounts number");
              }
              catch(Exception e)
              {
                  printStatusMsg(out,"Remote Exception occurred");
                  e.printStackTrace();
              }
          }
          else
          {
              printStatusMsg(out,"AccountManager object is null");
          }
           
          String dbstatus = "<code> Total accounts in database: " + size + "</code> ";
          out.println(dbstatus);
      }
      catch(Exception e)
      {
          printStatusMsg(out,"CAN NOT get database status");
          e.printStackTrace();
      }
      out.println("</body>");
      out.println("</html>");
   }

   ///////////////////////////////////////////////////////////////////////////////
   // Method: doAction1()
   ///////////////////////////////////////////////////////////////////////////////
   private void doAction1(PrintWriter out, String accountNumberToBeCreatedString)
   {    
       printActionResponse(out,action1Response,action1Text);  

       int accountNumberToBeCreated = 0;
       String debug = null;
       int accountNumber = 0;
       boolean isNumberValid = false;
       try
       {            
           accountNumber = read.parse(accountNumberToBeCreatedString).intValue();
           isNumberValid = true;
       }
       catch(ParseException e)
       {
           debug = "ParseException: " + e.toString();
       }
       catch(Exception ex)
       {
           debug = "Unexpected Exception: " + ex.toString();
       }
       if(debug!=null)
       {
           printStatusMsg(out,debug);
           String info =  invalidAccountNumberToBeCreatedMessage + " -> "+ accountNumberToBeCreatedString;
           printStatusMsg(out,info);
       }          

       if(isNumberValid) 
       {
           accountNumberToBeCreated = accountNumber;
           if(accountManager==null) this.getAccountManager();
           if(accountManager != null)
           {
               try
               {
                   int accountNumberCreated = accountManager.createAccounts(accountNumberToBeCreated);
                   if(accountNumberCreated < 0) printStatusMsg(out,action1ResultMessage2);
                   else printStatusMsg(out,(accountNumberCreated + " " + action1ResultMessage1));
               }
               catch(Exception e)
               {
                   printStatusMsg(out,(action1ResultMessage2 + " <br> " + e.toString()));
                   e.printStackTrace();
               }
           }
           else
           {
               printStatusMsg(out,(action1ResultMessage2 + " <br> AccountManager object is null"));
           }
       }
   }
   ///////////////////////////////////////////////////////////////////////////////
   // Method: doAction2()
   ///////////////////////////////////////////////////////////////////////////////
   private void doAction2(PrintWriter out, String balanceString)
   {    
       printActionResponse(out,action2Response,action2Text);  
       
       String debug = null;
       double minibalance = 0;
       boolean isBalanceValid = false;
       try
       {            
           minibalance = read.parse(balanceString).doubleValue();
           isBalanceValid = true;
       }
       catch(ParseException e)
       {
           debug = "ParseException: " + e.toString();
       }
       catch(Exception ex)
       {
           debug = "Unexpected Exception: " + ex.toString();
       }

       if(debug!=null)
       {
           printStatusMsg(out,debug);
           String info =  invalidBalanceMessage + " -> "+ balanceString;
           printStatusMsg(out,info);
       }          
       else printStatusMsg(out,(miniBalanceText + " = " + minibalance));

       if(isBalanceValid)
       {
           String[] accountRecords = null;
           if(accountManager==null) this.getAccountManager();
           if(accountManager != null)
           {
               try
               {
                   accountRecords = accountManager.getLargeAccounts(minibalance);
               }
               catch(Exception e)
               {
                   debug = "Exception in accountManager.getLargeAccounts(): " + e.toString();
                   printStatusMsg(out,debug);
                   e.printStackTrace();
               }
           }
           if(accountRecords!=null && accountRecords.length>0)
           {
               printStatusMsg(out,(accountRecords.length + " " + action2ResultMessage1));
               out.println("<table width=\"250\" border=\"1\">");
               out.println("<tr>");
               out.println("<td align=\"right\" width=\"100\">" + "  AccountId  " + "</td>");
               out.println("<td align=\"right\" width=\"100\">" + "  Balance  " + "</td>");
               out.println("<td align=\"right\" width=\"50\">" + "  APR  " + "</td>");
               out.println("</tr>");

               StringTokenizer token = null;
               String accountid = null;
               String balance = null;
               String apr = null;
               for(int i=0; i<accountRecords.length; i++)
               {
                   token = new StringTokenizer(accountRecords[i]);
                   if(token.hasMoreElements()) {
                       accountid = token.nextToken();
                       balance = token.nextToken();
                       apr = token.nextToken();
                   }
                   out.println("<tr>");
                   out.println("<td align=\"right\" width=\"100\">" + accountid + "</td>");
                   out.println("<td align=\"right\" width=\"100\">" + balance + "</td>");
                   out.println("<td align=\"right\" width=\"50\">" + apr + "</td>");
                   out.println("</tr>");
               }
               out.println("</table>");
           }
           else
           {
               printStatusMsg(out,action2ResultMessage2);
           }
       }
   }
   ///////////////////////////////////////////////////////////////////////////////
   // Method: doAction3()
   ///////////////////////////////////////////////////////////////////////////////
   private void doAction3(PrintWriter out, String minibalanceString)
   {    
       printActionResponse(out,action3Response,action3Text);  

       String debug = null;
       double minibalance = 0;
       boolean isBalanceValid = false;
       try
       {            
           minibalance = read.parse(minibalanceString).doubleValue();
           isBalanceValid = true;
       }
       catch(ParseException e)
       {
           debug = "ParseException: " + e.toString();
       }
       catch(Exception ex)
       {
           debug = "Unexpected Exception: " + ex.toString();
       }

       if(debug!=null)
       {
           printStatusMsg(out,debug);
           String info =  invalidBalanceMessage + " -> "+ minibalanceString;
           printStatusMsg(out,info);
       }          
       else printStatusMsg(out,(miniBalanceText + " = " + minibalance));

       if(isBalanceValid)
       {
           if(accountManager==null) this.getAccountManager();
           if(accountManager != null)
           {
               try
               {
                   int accountNumber = accountManager.increaseLargeAccountsAPR(minibalance,aprIncreaseRate);
                   if(accountNumber!=-1)
                   {
                       printStatusMsg(out,(accountNumber + " " + action3ResultMessage1));
                   }
                   else
                   {
                       printStatusMsg(out,action3ResultMessage2);
                   }
               }
               catch(Exception e)
               {
                   debug = "Exception in accountManager.increateLargeAccountsAPR(): " + e.toString();
                   printStatusMsg(out,debug);
                   e.printStackTrace();
               }
           }
       }
   }

   ///////////////////////////////////////////////////////////////////////////////
   // Method: doAction4()
   ///////////////////////////////////////////////////////////////////////////////
   private void doAction4(PrintWriter out)
   {    
       printActionResponse(out,action4Response,""); 
       try
       {
           this.deleteAllAccounts(out);
       
       } catch (SQLException sql) {
           printStatusMsg(out,("SQLException = " + sql.toString()));
       }

   }

   ///////////////////////////////////////////////////////////////////////////////
   // Method: printPageTemplate()
   ///////////////////////////////////////////////////////////////////////////////
   private void printPageTemplate(PrintWriter out)
   {    
       out.println("<script language=\"JavaScript\" src=\"/WSsamples/scripts/browser_detection.js\"></script>");

       out.println("<NOSCRIPT>");
       out.println("Proper display of samples requires JavaScript to be enabled.");
       out.println("</NOSCRIPT>");
   }


   ///////////////////////////////////////////////////////////////////////////////
   // Method: printPageTemplate()
   ///////////////////////////////////////////////////////////////////////////////
   private void printSampleTitle(PrintWriter out)
   {    
       out.println("<table cellpadding=5 cellspacing=5>");
       out.println("<tr><td>");
       out.println("<H2 class=\"sampjsp\">" + heading + " </H2>");
       out.println("</td></tr>");
       out.println("</table>");
       out.println("<HR width=100%>");
   }

   ///////////////////////////////////////////////////////////////////////////////
   // Method: printStatusMsg()
   ///////////////////////////////////////////////////////////////////////////////
   private void printStatusMsg(PrintWriter out, String msg)
   {    
       out.println("<code>" + msg + "</code><br>");
   }

   ///////////////////////////////////////////////////////////////////////////////
   // Method: printActionResponse()
   ///////////////////////////////////////////////////////////////////////////////
   private void printActionResponse(PrintWriter out, String actionResponse, String actionText)
   {    
       out.println("<b>" + actionResponse + "</b>" + actionText);
       out.println("<HR>");
   }

   ///////////////////////////////////////////////////////////////////////////////
   // Method: printMenu()
   ///////////////////////////////////////////////////////////////////////////////
   private void printMenu(PrintWriter out, String accountNumberToBeCreatedString, String balanceString, String minibalanceString)
   {
       if(accountNumberToBeCreatedString==null) accountNumberToBeCreatedString = "" + sampleAccountsNumber;
       if(balanceString==null) balanceString = defaultBalance;
       if(minibalanceString==null) minibalanceString = defaultMiniBalance;

       String formOpen = "<form method=\"get\" action=\"" + servletURL + "\" name=\"menu\">";
       String formClose = "</form>";

       String action1Button = "<P><input type=\"submit\" value=\"" + action1ButtonValue + "\" name=\"action1\"></P>";
       String action1TextBox =  "<input type=\"text\" name=\"accountNumberToBeCreated\" id=\"accountNumberToBeCreated\" size=\"10\" value=\"" + accountNumberToBeCreatedString + "\" maxlength=\"10\">"; 

       String action2Button = "<P><input type=\"submit\" value=\"" + action2ButtonValue + "\" name=\"action2\"></P>";
       String action2TextBox =  "<input type=\"text\" name=\"balance\" id=\"balance\" size=\"10\" value=\"" + balanceString + "\" maxlength=\"10\">"; 

       String action3Button = "<P><input type=\"submit\" value=\"" + action3ButtonValue + "\" name=\"action3\"></P>";
       String action3TextBox =  "<input type=\"text\" name=\"minibalance\" id=\"minibalance\" size=\"10\" value=\"" + minibalanceString + "\" maxlength=\"10\">"; 

       //Menu table
       out.println("<p>" + menuPrompt + "</p>");
       out.println("<p></p>");
       out.println(formOpen);
       out.println("<table border=\"0\"cellspacing=\"3\" cellpadding=\"3\" >");

       out.println("<tr>");
       out.println("<th>Activity</th>");
       out.println("<th colspan=2>Value</th>");
       out.println("<th>Action</th>");
       out.println("</tr>");

       out.println("<tr>");
       out.println("<td align=\"left\"><b>1. " + action1Text + "</b></td>");
       out.println("<label for=\"accountNumberToBeCreated\">");
       out.println("<td align=\"left\">" + action1Description + "</td>");
       out.println("<td align=\"left\">" + action1TextBox + "</td>");
       out.println("</label>");
       out.println("<td align=\"right\">" + action1Button + "</td>");
       out.println("</tr>");
       out.println("<tr>");
       out.println("<td align=\"left\"><b>2. " + action2Text + "</b></td>");
       out.println("<label for=\"balance\">");
       out.println("<td align=\"left\">" + action2Description + "</td>");
       out.println("<td align=\"left\">" + action2TextBox + "</td>");
       out.println("</label>");
       out.println("<td align=\"right\">" + action2Button + "</td>");
       out.println("</tr>");
       out.println("<tr>");
       out.println("<td align=\"left\"><b>3. " + action3Text + "</b></td>");
       out.println("<label for=\"minibalance\">");
       out.println("<td align=\"left\">" + action3Description + "</td>");
       out.println("<td align=\"left\">" + action3TextBox + "</td>");
       out.println("</label>");
       out.println("<td align=\"right\">" + action3Button + "</td>");
       out.println("</tr>");
       out.println("</table>");
       out.println(formClose);
   }

    private Connection makeConnection() throws SQLException {
    	if (ds == null) {
    		try {
    			if (this.initCtx==null) initCtx = new InitialContext();
    			ds = (DataSource) initCtx.lookup(datasourceBinding);
    		} catch(Exception e) {
                    e.printStackTrace();
    		}
    	}
        Connection conn = null;
        if(ds != null) conn = ds.getConnection();
    	return conn;
    }

    private Collection getAllAccountid() throws SQLException {
    	Connection con = makeConnection(); 
    	ArrayList results = new ArrayList();
    	String selectStatement = "SELECT * FROM " + tableName;
    
    	PreparedStatement prepStmt = null;
    	ResultSet rs = null;

    	try {
            prepStmt = con.prepareStatement(selectStatement);    
            rs = prepStmt.executeQuery();    
            while(rs.next()) {
                    String accountid = new String((rs.getObject(1)).toString());
                    results.add(accountid);
            }
    	} catch (Exception e) {
            e.printStackTrace();
    	} finally {
            if (rs!=null) rs.close();
            if (prepStmt!=null) prepStmt.close();
            if (con!=null) con.close();
    	}    
    	return results;
    }


    private void deleteAllAccounts(PrintWriter out) throws SQLException {
    	Connection con = makeConnection(); 
    
    	String deleteStatement = "DELETE FROM " + tableName +
    		" WHERE type = 1";
    
        printStatusMsg(out,"SQL statement = " + deleteStatement);
    	PreparedStatement prepStmt = null;
    	try {
            prepStmt = con.prepareStatement(deleteStatement);    
    	    prepStmt.executeUpdate();
    	} catch (SQLException sql) {
            printStatusMsg(out,("SQLException = " + sql.toString()));
    	} finally {
    	    if (prepStmt!=null) prepStmt.close();
    	    if (con!=null) con.close();
    	}

        Collection accountIdCollection = null;
        try
        {
            accountIdCollection = this.getAllAccountid();
            int size = accountIdCollection.size();
            printStatusMsg(out,("Total accounts after clean up database: " + size));
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    ///////////////////////////////////////////////////////////////////////////////
    // Method: getAccountManager()
    ///////////////////////////////////////////////////////////////////////////////
    private void getAccountManager() {

        if(this.initCtx==null)
        {
            try
            {
                initCtx = new InitialContext();
            }
            catch(Exception e)
            {
                e.printStackTrace();
            }
        }

        try
        {
            Object o = initCtx.lookup(this.accountManagerName);
            if (o == null) {
                    throw new Exception("ERROR: lookup failed");
            }
            AccountManagerHome home = 
                (AccountManagerHome) javax.rmi.PortableRemoteObject.narrow(o,AccountManagerHome.class); 

            try
            {
                accountManager = (AccountManager) home.create();
            }            
            catch (Exception e )
            {
                e.printStackTrace();
                accountManager = null;
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
    * This method replaces characters with the equivalent HTML escape syntax.
	* The characters are: double quote, less than, greater than. 
	* 
	* @param input text from an input field.
	* 
	* @return text with HTML escape syntax.
	*/
    private String replaceSpecialCharacters(String input)
    {
        if (input == null) {
            return "";
        }

        return input.replaceAll("<", "&lt;").replace(">", "&gt;").replace("\"", "&quot;");
    }
}
