//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  COPYRIGHT International Business Machines Corp., 2001,2008
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.scheduler.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Locale;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.scheduler.accountreportejb.Account;
import com.ibm.websphere.samples.scheduler.accountreportejb.AccountHome;
import com.ibm.websphere.samples.scheduler.accountreportejb.AccountReport;
import com.ibm.websphere.samples.scheduler.accountreportejb.AccountReportHome;
import com.ibm.websphere.samples.scheduler.accountreportejb.TaskCreator;
import com.ibm.websphere.samples.scheduler.accountreportejb.TaskCreatorHome;
import com.ibm.websphere.scheduler.TaskInvalid;
import com.ibm.websphere.scheduler.UserCalendarInvalid;
import com.ibm.websphere.scheduler.UserCalendarPeriodInvalid;

/**
 * The Scheduler Sample Account Report Servlet.  Allows
 * scheduling tasks to create reports based on total account balances.
 */
public class Main extends javax.servlet.http.HttpServlet 
{
    /**
     * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response) 
        throws ServletException, IOException
    {
        doProcess(request, response);
    }
    
    /**
     * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest, HttpServletResponse)
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        doProcess(request, response);
    }

    /**
     * Process all HTML requests
     */
    private void doProcess(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        PrintWriter writer = response.getWriter();
        
        try {
            if (!response.isCommitted()) 
            {
                response.setContentType("text/html");
            }
            
            printHeading(writer);
            
            // Format our balances based on current locale.
            Locale locale = request.getLocale();
            NumberFormat numberFormat = DecimalFormat.getInstance(locale);
            numberFormat.setMinimumFractionDigits(2);
            numberFormat.setMaximumFractionDigits(2);
            
            try
            {
                // Schedule a task
                String startInterval = "0";
                int repeats = 5;
                String repeatsStr = "";
                String repeatInterval = "60";
                Throwable taskErrorDetected = null;
                String temp = null;
                try 
                {
                    // Assemble our CalendarDelta strings from the input parameters where necessary
                    temp = request.getParameter("startint");
                    if (temp != null)
                        startInterval = temp;

                    temp = request.getParameter("repeatint");
                    if (temp != null)
                        repeatInterval = temp;
                        
                    temp = request.getParameter("repeats");
                    if (temp != null)
                    {
                        repeatsStr = temp;
                        repeats = Integer.parseInt(temp);
                    }
                }
                catch(Throwable e)
                {   taskErrorDetected = e;
                }
    
                printSchedTaskForm(writer, startInterval, repeatsStr, repeatInterval);
                
                if ((request.getParameter("ACTION") != null) && (request.getParameter("ACTION").equals("Schedule")))
                {
                    // We only accept positive values in this sample to prevent running tasks indefinitely
                    if (repeats < 0) 
                    {
                        printErrorMessage(writer, "Only positive repeat intervals are valid for this sample.  Negative values will normally cause the task to repeat indefinitely.  Please try again.");
                    }
                    // Attempt to schedule the task
                    else  
                    {
                        if (taskErrorDetected == null)
                        {
                            scheduleTask(startInterval, repeats, repeatInterval);
                            
                            printResultMessage(writer, "A new report will be generated in " + replaceSpecialCharacters(startInterval) + " seconds.");
                            if (repeatInterval.trim().length() != 0 && !repeatInterval.trim().equals("0"))
                            {
                                printResultMessage(writer, "A total of " + repeats + " reports will be generated with an interval of " + replaceSpecialCharacters(repeatInterval) + " seconds between each report.");
                            }
                            printResultMessage(writer, "Note the <samp>Poll Interval</samp> defined in the Scheduler resource will be the effective minimum repeat interval (30 seconds for this sample).");
                        }
                        else
                        {
                            handleException(writer, taskErrorDetected);
                        }
                    }
                }
            }
            catch(Throwable e)
            {
                handleException(writer, e);
            }

            try
            {
                // Add an account
                Throwable accountErrorDetected = null;
                String accountStr = "";
                Integer account = new Integer(0);
                String balanceStr = "";
                Double balance = new Double(0);
                String owner = "";
                String desc = "";
                String temp = null;
                try
                {
                    temp = request.getParameter("acctnum");
                    if (temp != null)
                    {
                        accountStr = temp;
                        account = new Integer(Integer.parseInt(temp));
                    }
                }
                catch(Throwable e)
                {
                    if (accountErrorDetected == null) accountErrorDetected = e;
                }
                try
                {
                    temp= request.getParameter("balance");
                    if(temp != null)
                    {
                        balanceStr = temp;
                        balance = new Double(numberFormat.parse(temp).doubleValue());
                    }
                }
                catch(Throwable e)
                {
                    if (accountErrorDetected == null) accountErrorDetected = e;
                }

                temp = request.getParameter("owner");
                if (temp != null)
                    owner = temp;

                temp = request.getParameter("desc");
                if (temp != null)
                    desc = temp;
    
                printNewAccountForm(writer, accountStr, balanceStr, owner, desc);

                if ((request.getParameter("ACTION") != null) && (request.getParameter("ACTION").equals("Create Account")))
                {
                    if (accountErrorDetected == null)
                    {
    
                        addAccount(account, balance, owner, desc);
        
                        printResultMessage(writer, "A new account was created with account number " + account + " with a balance of " + numberFormat.format(balance));
                    }
                    else 
                    {
                        handleException(writer, accountErrorDetected);
                    }
    
                }
            }
            catch(Throwable e)
            {
                handleException(writer, e);
            }

            printAccounts(writer, locale);
                    
            try
            {    
                // Report form
                printReportsForm(writer);
                printReports(writer, locale);
            }
            catch(Throwable e)
            {
                handleException(writer, e);
            }
        } 
        catch(Throwable e) 
        {            
            handleException(writer, e);
        } 
        finally 
        {
            
            printFooter(writer);
        }
        
        
    }


    private void handleException(PrintWriter writer, Throwable e) throws IOException
    {
        if (e instanceof TaskInvalid)
        {
            printErrorMessage(writer, "The task threw a TaskInvalid exception with the following message:  " + e.getMessage() + ".  Please correct the error and try again.");
        }
        else if (e instanceof NumberFormatException)
        {
            printErrorMessage(writer, "There was an error parsing a number. Please correct the error and try again.");
            printErrorMessage(writer, "The number of repeats is any positive, non-zero integer value.");
            printErrorMessage(writer, "The account number is any integer value.");
            printErrorMessage(writer, "The balance is any decimal number formatted using your browser's locale.");
        }
        else if (e instanceof ParseException)
        {
            printErrorMessage(writer, "There was an error parsing some of the entered data.  Please correct the error and try again.");
        }
        else if (e instanceof UserCalendarPeriodInvalid)
        {
            printErrorMessage(writer, "The task validator threw a UserCalendarPeriodInvalid exception with the following message: " + e.getMessage() + ".  Please correct the error and try again.");
        }
        else if (e instanceof UserCalendarInvalid)
        {
            printErrorMessage(writer, "The task validator threw a UserCalendarInvalid exception with the following message: " + e.getMessage() + ".  Please correct the error and try again.");
        }
        else if (e instanceof DuplicateKeyException)
        {
            printErrorMessage(writer, "The account number you entered is not unique.  Please correct the error and try again.");
        }
        else
        {
            writer.println("<font color=\"red\">");
            writer.println("An unexpected exception occurred.  Please correct the error and try again:");
            writer.println("<pre>");
            e.printStackTrace(writer);
            writer.println("</pre>");
            writer.println("</font>");
        } 
    }
    
    /**
     * Schedule a task using the TaskCreator EJB.  Each time that the task is executed, it will generate a report
     * with the current account totals in it.
     * @param startInterval The interval to wait for the first task to execute.  This is in default calendar delta format.
     * @param repeats The number of times to repeat.  A value <= 0 means that it will continue indefinitely
     * @param repeatInterval The amount of time to wait between repeats.  This is in default calendar delta format.
     */
    protected void scheduleTask(String startInterval, int repeats, String repeatInterval)
        throws Exception
    {
        // Create a Schedule Task
        
        // Look-up the TaskCreatorBean
        try
        {
            InitialContext ic = new InitialContext();
            Object taskCreatorHomeObj = ic.lookup("java:comp/env/ejb/TaskCreator");
            TaskCreatorHome taskCreatorHome = (TaskCreatorHome) javax.rmi.PortableRemoteObject.narrow(taskCreatorHomeObj, TaskCreatorHome.class);
            
            TaskCreator taskCreator = taskCreatorHome.create();
            taskCreator.scheduleTask(startInterval, repeats, repeatInterval);
            
        } 
        catch (NamingException e)
        {
            throw e;
        }
        catch (RemoteException e)
        {
            throw e;
        }
        catch (CreateException e)
        {
            throw e;
        }
    }

    /**
     * Add a new account to the database.
     * @param accountNumber any unique number.
     * @param newBalance an amount to allow the scheduler task to total
     * @param ownerName any string
     * @param desc any string
     * @throws Exception
     */
    protected void addAccount(Integer accountNumber, Double newBalance, String ownerName, String desc)
        throws Exception
    {
        // Add an account 
        
        try
        {
            InitialContext ic = new InitialContext();
            Object accountHomeObj = ic.lookup("java:comp/env/ejb/Account");
            AccountHome accountHome = (AccountHome) javax.rmi.PortableRemoteObject.narrow(accountHomeObj, AccountHome.class);
            
            Account account = accountHome.create(accountNumber);
            account.setBalance(newBalance);
            account.setOwnerName(ownerName);
            account.setDescription(desc);
            
        } 
        catch (NamingException e)
        {
            throw e;
        } 
        catch (RemoteException e)
        {
            throw e;
        }
        catch (CreateException e)
        {
            throw e;
        }
    }

    protected void printHeading(PrintWriter writer)
        throws IOException
    {
        writer.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
        writer.println("<!--");
        writer.println("\"This program may be used, executed, copied, modified and distributed without royalty for the purpose of developing, using, marketing, or distributing.\"");
        writer.println();
        writer.println("Product 5639-D57, 5724i63, 5630-A37, 5724-D18,  (C) COPYRIGHT International Business Machines Corp., 2001, 2004");
        writer.println("All Rights Reserved * Licensed Materials - Property of IBM");
        writer.println("-->");

        writer.println("<HTML lang=\"en\">");
        writer.println("<HEAD>");
        writer.println("<META http-equiv=\"Content-Type\" CONTENT=\"text/html; charset=iso-8859-1\">");
        writer.println("<TITLE>Scheduler Account Report Sample</TITLE>");
        writer.println("<script language=\"JavaScript\" src=\"/WSsamples/scripts/browser_detection.js\"></script>");

        writer.println("<NOSCRIPT>");
        writer.println("Proper display of samples requires JavaScript to be enabled.");
        writer.println("</NOSCRIPT>");

        writer.println("</HEAD>");
        writer.println("<BODY leftmargin=0 topmargin=0 marginwidth=0 marginheight=0>");
        writer.println("<FORM>");

        // Show the menu
        writer.println("<img src=\"/WSsamples/images/success_banner.jpg\" valign=top alt=\"WebSphere Samples\">");

        writer.println("<TABLE summary=\"This table provides layout for the entire Account Report page.\"><TR><TD>");
        writer.println("<H2>Account Report Sample</H2>");

        writer.println("<UL>");
        writer.println("<LI>To schedule a new task, enter values, then click the Schedule button</LI>");
        writer.println("<LI>To create a new account, enter values, then click the Create Account button</LI>");
        writer.println("<LI>To refresh the reports which have been generated, just click the refresh button on your web browser</LI>");
        writer.println("</UL>");
    }

    protected void printFooter(PrintWriter writer)
        throws IOException
    {
        writer.println("</FORM></TD></TR></TABLE><P></P>");
        writer.println("</BODY>");
        writer.println("</HTML>");
    }

    protected void printSchedTaskForm(PrintWriter writer, String startInterval, String repeats, String repeatInterval)
        throws IOException
    {
        writer.println("<HR>");
        writer.println("<H3>Schedule Task</H3>");
        writer.println("<TABLE summary=\"This table provides the layout for the Schedule Task form.\">");
        writer.println("<TR><TD><LABEL FOR=\"startint\">Start Time (seconds)</LABEL>:</TD><TD><INPUT TYPE=\"TEXT\" name=\"startint\" value=\"" + replaceSpecialCharacters(startInterval) + "\" id=\"startint\" SIZE=\"4\"/></TD</TR>");
        writer.println("<TR><TD><LABEL FOR=\"repeats\">Number of Repeats</LABEL>:</TD><TD><INPUT TYPE=\"TEXT\" name=\"repeats\" id=\"repeats\" value=\"" + replaceSpecialCharacters(repeats) + "\" SIZE=\"4\"/></TD></TR>");
        writer.println("<TR><TD><LABEL FOR=\"repeatint\">Interval Between Repeats (seconds)</LABEL>:</TD><TD><INPUT TYPE=\"TEXT\" name=\"repeatint\" id=\"repeatint\" value=\"" + replaceSpecialCharacters(repeatInterval) + "\" SIZE=\"4\"/></TD></TR>");
        writer.println("<TR><TD><INPUT TYPE=\"SUBMIT\" NAME=\"ACTION\" VALUE=\"Schedule\"></TD></TR>");
        writer.println("</TABLE>");
    }
    
    protected void printNewAccountForm(PrintWriter writer, String acctnum, String balance, String owner, String description)
        throws IOException
    {
        writer.println("<HR>");
        writer.println("<H3>New Account</H3>");
        writer.println("<TABLE summary=\"This table provides the layout for the New Account form.\">");
        writer.println("<TR><TD><LABEL FOR=\"acctnum\">Account Number:</LABEL></TD><TD><INPUT TYPE=\"TEXT\" name=\"acctnum\" id=\"acctnum\" value=\"" + replaceSpecialCharacters(acctnum) + "\" /></TD></TR>");
        writer.println("<TR><TD><LABEL FOR=\"balance\">Balance:</LABEL></TD><TD><INPUT TYPE=\"TEXT\" name=\"balance\" id=\"balance\" value=\"" + replaceSpecialCharacters(balance) + "\" /></TD><TR>");
        writer.println("<TR><TD><LABEL FOR=\"owner\">Owner:</LABEL></TD><TD><INPUT TYPE=\"TEXT\" name=\"owner\" id=\"owner\" value=\"" + replaceSpecialCharacters(owner) + "\" /><TD></TD><TR>");
        writer.println("<TR><TD><LABEL FOR=\"desc\">Description:</LABEL></TD><TD><INPUT TYPE=\"TEXT\" SIZE=\"100\" name=\"desc\" id=\"desc\" value=\"" + replaceSpecialCharacters(description) + "\"/></TR>");
        writer.println("<TR><TD><INPUT TYPE=\"SUBMIT\" NAME=\"ACTION\" VALUE=\"Create Account\"></TD></TR>");
        writer.println("</TABLE>");
    }

    protected void printReportsForm(PrintWriter writer)
        throws IOException
    {
        writer.println("<HR>");
        writer.println("<H3>Reports</H3>");
        writer.println("<TABLE summary=\"This table provides the layout for the Reports form.\">");
        writer.println("<TR><TD>This table shows all of the reports that have been generated by the Task Scheduler:");
        writer.println("        <UL>");
        writer.println("        <LI><samp>Time Report Generated</samp> is the time which the report ");
        writer.println("          was generated by the Task Scheduler. ");
        writer.println("        <LI><samp>Total Balance</samp> is the sum of all account balances ");
        writer.println("          at the time the report was generated.");
        writer.println("       </UL></TD></TR>");
        writer.println("<TR><TD>Click the Refresh button to refresh the table contents.</TD></TR>");
        writer.println("<TR><TD><INPUT TYPE=\"SUBMIT\" NAME=\"ACTION\" VALUE=\"Refresh\"></TD></TR></TABLE>");
    }
    
    /**
     * Show all of the accounts that have been added
     * @param writer the HTTP response object's PrintWriter.
     * @param locale the local of the HTTP client to allow for formatting numbers.
     */
    protected void printAccounts(PrintWriter writer, Locale locale)
        throws Exception
    {
        InitialContext initialContext = new InitialContext();
        
        Object accountHomeObj = initialContext.lookup("java:comp/env/ejb/Account");
        AccountHome accountHome = (AccountHome) PortableRemoteObject.narrow(accountHomeObj, AccountHome.class);
        Collection accounts = accountHome.findAllAccounts();
        Iterator iterator = accounts.iterator();
        
        if(!iterator.hasNext())
        {
            printResultMessage(writer, "No accounts available.");
        }
        
        else
        {
            NumberFormat numberFormat = DecimalFormat.getInstance(locale);
            numberFormat.setMinimumFractionDigits(2);
            numberFormat.setMaximumFractionDigits(2);
            writer.println("<TABLE summary=\"This table shows all of the accounts that have been added with the New Account form.\" cellpadding=3 cellspacing=2 >");
            writer.println("<TR><TH>Account Number</TH><TH>Balance</TH><TH>Owner</TH><TH>Description</TH></TR>");

            // Sum-up the account balance of all of the accounts
            while (iterator.hasNext()) 
            {
                Account account = (Account)iterator.next();
                writer.println("<TR><TD>" + account.getAccountNumber() + "</TD><TD align=\"right\">" + numberFormat.format(account.getBalance()) + "</TD><TD>" + replaceSpecialCharacters(account.getOwnerName()) + "</TD><TD>" + replaceSpecialCharacters(account.getDescription()) + "</TD></TR>");
            }

            writer.println("</TABLE>");
        }
    }
    
    /**
     * Show all of the reports that have run
     * @param writer the HTTP response object's PrintWriter.
     * @param locale the local of the HTTP client to allow for formatting numbers.
     */
    protected void printReports(PrintWriter writer, Locale locale)
        throws Exception
    {
        try
        {
            // Lookup our AccountReport EJB.
            InitialContext ic = new InitialContext();
            Object accountReportHomeObj = ic.lookup("java:comp/env/ejb/AccountReport");
            AccountReportHome accountReportHome = (AccountReportHome)javax.rmi.PortableRemoteObject.narrow(accountReportHomeObj,AccountReportHome.class);    
            
            // Execute a finder method that will return a collection of AccountReport objects.
            Collection reports = accountReportHome.findAllReports();
            Iterator iterator = reports.iterator();
            
            if(!iterator.hasNext())
            {
                printResultMessage(writer, "No reports available.");
            }
            
            else
            {
                // Print a record for each AccountReport
                NumberFormat numberFormat = NumberFormat.getNumberInstance(locale);
                DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, locale);
                
                writer.println("<TABLE summary=\"This table shows all of the reports that have been generated by the Task Scheduler.\" cellpadding=3 cellspacing=2 >");
                writer.println("<TR><TH>Time Report Generated</TH><TH>Total Balance</TH></TR>");
                
                while(iterator.hasNext())
                {
                    AccountReport accountReport = (AccountReport)javax.rmi.PortableRemoteObject.narrow(iterator.next(),AccountReport.class);
                    
                    Date rptTime = new Date(accountReport.getTimeReportGenerated().longValue());
                    
                    writer.println("<TR><TD>" + dateFormat.format(rptTime) + "</TD><TD>" + numberFormat.format(accountReport.getTotalBalance()) + "</TD></TR>");
                }
    
                writer.println("</TABLE>");
            }
        } 
        catch (RemoteException e)
        {
            throw e;
        } 
        catch (NamingException e)
        {
            throw e;
        } 
        catch (FinderException e)
        {
            throw e;
        } 
        catch (IOException e)
        {
            throw e;
        }
        finally
        {
        }
        
        
    }
    
    protected void printResultMessage(PrintWriter writer, String message)
        throws IOException
    {
        writer.println("<p>" + replaceSpecialCharacters(message) + "</p>");
    }

    protected void printErrorMessage(PrintWriter writer, String message)
        throws IOException
    {
        writer.println("<font color=\"red\"><p>" + replaceSpecialCharacters(message) + "</p></font>");
    }

    /**
    * This method replaces characters with the equivalent HTML escape syntax.
	* The characters are: double quote, less than, greater than. 
	* 
	* @param input text from an input field.
	* 
	* @return text with HTML escape syntax.
	*/
    private String replaceSpecialCharacters(String input)
    {
	    return input.replaceAll("<", "&lt;").replaceAll(">", "&gt;").replaceAll("\"", "&quot;");
    }

}
